// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include <unistd.h>

#include "target_new_source.h"
#include "error.h"
#include "exec.h"
#include "log.h"
#include "picio.h"
#include "utils.h"

target_new_source::target_new_source(const std::string & id, const std::string & descr, source *const s, const double interval, const std::vector<filter *> *const filters, configuration_t *const cfg, const std::string & new_s_id, const std::string & new_s_descr) : target(id, descr, s, "", "", "", -1, interval, filters, "", "", "", -1, cfg, false, false), new_s_id(new_s_id), new_s_descr(new_s_descr)
{
}

target_new_source::~target_new_source()
{
	stop();
}

source * target_new_source::get_new_source()
{
	new_source_lock.lock();

	if (!new_source) {
		log(LL_INFO, "Instantiating new source %s", new_s_id.c_str());

		new_source = new source(new_s_id, new_s_descr, "", -1.0, nullptr, -1, -1, LL_DEBUG, 0.1, nullptr);
	}

	new_source_lock.unlock();

	return new_source;
}

void target_new_source::operator()()
{
	log(LL_INFO, "\"as-a-new-source\" thread started");

	set_thread_name("newsrc" + new_s_id);

	uint8_t *prev_frame = nullptr;
	uint64_t prev_ts = 0;

	s -> register_user();

        for(;!local_stop_flag;) {
                pauseCheck();

                double before_ts = get_ts();

		if (new_source) {
			int w = -1, h = -1;
			uint8_t *work = nullptr;
			size_t work_len = 0;
			uint64_t ts = prev_ts;
			if (s -> get_frame(E_RGB, -1, &ts, &w, &h, &work, &work_len, false)) {
				if (new_source->get_width() != w || new_source->get_height() != h)
					new_source->set_size(w, h);

				instance_t *inst = find_instance_by_interface(cfg, s);
				apply_filters(inst, s, filters, prev_frame, work, ts, w, h);

				new_source->set_frame(E_RGB, work, work_len);

				free(prev_frame);
				prev_frame = work;

				prev_ts = ts;
			}
		}

		handle_fps(before_ts);
	}

	free(prev_frame);

	s -> unregister_user();

	log(LL_INFO, "\"as-a-new-source\" thread terminating");
}
